package uwe;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 * This class is just necessary because intermediate results have to be stored
 * in Strings or List of Strings in XPand. If performance will play a role in
 * the future, keep this in mind!
 * 
 * @author Marianne Busch <busch@pst.ifi.lmu.de>
 * 
 */
public class FormatTransformationOutput {

	private static Integer increasingNumber = 0;

	/**
	 * @param rawPermissions
	 * @return List of TargetName1[permission1#permission2]
	 */
	public static List<String> getPermissionsGroupedByTarget(List<String> rawPermissions) {
		Map<String, List<String>> map = new HashMap<>();
		for (String p : rawPermissions) {
			String key = getTargetName(getResources(p));
			List<String> list = map.get(key);
			if (list == null) {
				list = new LinkedList<>();
				list.add(p);
				map.put(key, list);
			} else {
				list.add(p);
				map.put(key, list);
			}
		}
		List<String> groupedPermissions = new LinkedList<String>();
		for (String targetKey : map.keySet()) {
			StringBuilder l = new StringBuilder("");
			for (String p : map.get(targetKey)) {
				l.append(p);
				l.append("#");
			}
			groupedPermissions.add(targetKey + "[" + l.toString().substring(0, l.length() - 1) + "]");
		}
		return groupedPermissions;
	}

	public static List<String> getPermissionsOnly(String permissionGroup) {
		List<String> permissionsOnly = new LinkedList<>();
		String result = permissionGroup.split("\\[")[1];
		result = result.substring(0, result.length()-1);
		for (String p : result.split("#")) {
			permissionsOnly.add(p);
		}
		return permissionsOnly;
	}

	public static List<String> getResources(String permission) {
		List<String> resources = new LinkedList<>();
		for (String resource : permission.split(": ")[1].split(", ")) {
			if (!resource.startsWith("_constraint_")) {
				resources.add(resource);
			}
		}
		return resources;
	}

	static String getTargetName(List<String> resources) {
		for (String p : resources) {
			if (p.contains(".")) {
				return p.split("\\.")[0];
			} else {
				return p;
			}
		}
		return null;
	}

	public static String getResourceWithoutPrefix(String resource) {
		if (resource.contains(".")) {
			return resource.split("\\.")[1];
		} else {
			return resource;
		}
	}

	public String getActionFromPermissionString(String permission) {
		return permission.split(": ")[0];
	}

	public String getActionOnlyFromPermissionString(String permission) {
		String action = getActionFromPermissionString(permission);
		return action.contains("__") ? action.split("__")[0] : action;
	}

	public static List<String> getConstraint(String constraint) {
		List<String> constraints = new LinkedList<>();
		// FIXME: parse OCL correctly!
		final String integerEqual = "<Apply FunctionId=\"urn:oasis:names:tc:xacml:1.0:function:integer-equal\">\n";
		final String datatypeInt = "DataType=\"http://www.w3.org/2001/XMLSchema#integer\"";
		final String attribute1 = "<SubjectAttributeDesignator AttributeId=\"self.id\" " + datatypeInt + "/>";
		final String attribute2 = "<SubjectAttributeDesignator AttributeId=\"caller.id\" " + datatypeInt + "/>";
		StringBuilder result = new StringBuilder();
		result.append("<Condition>\n");
		switch (constraint) {
		case "caller.id = self.id":
			result.append(integerEqual);
			result.append(attribute1);
			result.append(attribute2);
			break;
		case "caller.id &lt;&gt; self.id":
			result.append("<Apply FunctionId=\"urn:oasis:names:tc:xacml:1.0:function:not\">");
			result.append(integerEqual);
			result.append(attribute1);
			result.append(attribute2);
			result.append("</Apply>");
			break;
		default:
			UWE2XACML.logger.fatal("Cannot parse OCL constraint: \"" + constraint + "\"");
			return null;
		}
		result.append("</Apply>\n</Condition>");
		constraints.add(result.toString());
		return constraints;
	}
	
	public static Integer getNumber(){
		increasingNumber = increasingNumber + 1;
		return increasingNumber;
	}
}
